@@ -43,13 +43,13 @@ PLATFORMS = {
 
 
 
-def diffconfig(diff, devicename, from_filename, to_filename, output_filename,
+def diffconfig(diff, devicename, old_filename, new_filename, output_filename,
                no_output, dump_rules, debug_parser, debug_convert):
 
-    """This function compares the 'from' and 'to' configurations for
-    the specified device and writes a difference configuration file
-    (one that transforms the configuration of a running device from the
-    'from' to the 'to' state).
+    """This function compares the old and new configurations for the
+    specified device and writes a difference configuration file (one
+    that transforms the configuration of a running device from the old
+    to the new state).
 
     The filenames for the configuration files, as well as the output
     are taken from the arguments parsed above and stored in global
@@ -67,9 +67,9 @@ def diffconfig(diff, devicename, from_filename, to_filename, output_filename,
     devicename -- the name of the device being converted, or None if
     it's not known
 
-    from_filename -- filename of the source configuration
+    old_filename -- filename of the source configuration
 
-    to_filename -- filename of the target configuration
+    new_filename -- filename of the target configuration
 
     output_filename -- name of the file to write the conversion
     configuration to or None for standard output
@@ -99,12 +99,12 @@ def diffconfig(diff, devicename, from_filename, to_filename, output_filename,
             num += 1
 
 
-    # read in the 'from' and 'to' configurations
+    # read in the old and new configurations
 
-    from_cfg = config_parser_class(filename=from_filename, debug=debug_parser)
+    from_cfg = config_parser_class(filename=old_filename, debug=debug_parser)
 
-    to_cfg = (config_parser_class(filename=to_filename, debug=debug_parser)
-                  if to_filename else None)
+    to_cfg = (config_parser_class(filename=new_filename, debug=debug_parser)
+                  if new_filename else None)
 
 
     # find the differences
@@ -237,7 +237,7 @@ parser.add_argument(
     default=0,
     help="increase debugging of the difference converstion action "
          "processing (multiple uses increase verbosity up to a "
-         "maximum of 3)")
+         "maximum of 4)")
 
 parser.add_argument(
     "platform",
@@ -245,17 +245,17 @@ parser.add_argument(
     help="platform used for configuration files")
 
 parser.add_argument(
-    "from_filename",
-    metavar="from",
-    help="initial ('from') configuration file; '%%' can be used to "
-         "substitute in the name of the device into the filename")
+    "old_filename",
+    metavar="old",
+    help="source (old / 'from') configuration file; '%%' can be used "
+         "to substitute in the name of the device into the filename")
 
 parser.add_argument(
-    "to_filename",
+    "new_filename",
     nargs="?",
     default=None,
-    metavar="to",
-    help="destination ('to') configuration file; '%%' can be used to "
+    metavar="new",
+    help="target (new / 'to') configuration file; '%%' can be used to "
          "substitute in the name of the device into the filename; if "
          "omitted just parse the file and assume 'debug' mode")
 
@@ -298,8 +298,8 @@ dump_diff_tree = args.dump_diff_tree
 subtree_dump_filter = args.subtree_dump_filter
 debug_parser = args.debug_parser
 debug_convert = args.debug_convert
-from_filename = args.from_filename
-to_filename = args.to_filename
+old_filename = args.old_filename
+new_filename = args.new_filename
 output_filename = args.output_filename
 devicenames = args.devicenames
 
@@ -321,17 +321,17 @@ if not rule_specs:
 # to multiple devices
 
 if not len(devicenames):
-    if from_filename.find("%") != -1:
+    if old_filename.find("%") != -1:
         print("error: no device names specified, so operating on a single "
-              "file, yet 'from' filename has '%' character",
+              "file, yet old filename has '%' character",
               file=sys.stderr)
 
         exit(1)
 
 
-    if to_filename and (to_filename.find("%") != -1):
+    if new_filename and (new_filename.find("%") != -1):
         print("error: no device names specified, so operating on a single "
-              "file, yet 'to' filename has '%' character",
+              "file, yet new filename has '%' character",
               file=sys.stderr)
 
         exit(1)
@@ -346,13 +346,13 @@ if not len(devicenames):
 
 
 elif len(devicenames) > 1:
-    if from_filename.find("%") == -1:
-        print("warning: multiple device names specified but 'from' filename "
+    if old_filename.find("%") == -1:
+        print("warning: multiple device names specified but old filename "
               "does not contain '%' - same file will be read",
               file=sys.stderr)
 
-    if to_filename and (to_filename.find("%") == -1):
-        print("warning: multiple device names specified but 'to' filename "
+    if new_filename and (new_filename.find("%") == -1):
+        print("warning: multiple device names specified but new filename "
               "does not contain '%' - same file will be read",
               file=sys.stderr)
 
@@ -425,8 +425,8 @@ if devicenames:
 
         diffs_tree = diffconfig(
                       diff, devicename,
-                      from_filename.replace("%", devicename),
-                      to_filename.replace("%", devicename),
+                      old_filename.replace("%", devicename),
+                      new_filename.replace("%", devicename),
                       output_filename.replace("%", devicename),
                       no_output, dump_rules, debug_parser,
                       debug_convert)
@@ -439,7 +439,7 @@ if devicenames:
 
 else:
     all_diffs_tree = diffconfig(
-        diff, None, from_filename, to_filename, output_filename, no_output,
+        diff, None, old_filename, new_filename, output_filename, no_output,
         dump_rules, debug_parser, debug_convert)
 
 
