@@ -19,16 +19,30 @@ differences between two configurations.
 
 from copy import deepcopy
 from itertools import chain
-import re
 import sys
 
 from deepops import (
-    deepdiff, deepfilter, deepget, deepmerge, deepremoveitems, deepsetdefault)
+    deepdiff, deepfilter, deepget, deepremoveitems, deepsetdefault)
 
 import yaml
 
 
 
+# --- constants ---
+
+
+
+# different debugging levels and what they mean (they're used in lots of
+# places so it seems sensible to avoid hard coding the values, in case
+# anything changes)
+
+DEBUG_CONVERT_STEPS = 1
+DEBUG_CONVERT_PARAMS = 2
+DEBUG_CONVERT_NODIFF = 3
+DEBUG_CONVERT_NOMATCH = 4
+
+
+
 # --- functions ---
 
 
@@ -61,9 +75,9 @@ def pathstr(path, wildcard_indices=set()):
 
 
 class DiffConvert:
-    """This abstract class handles converting the difference between a
-    'from' configuration item to the corresponding 'to' configuration
-    item.
+    """This abstract class handles converting the difference between an
+    old ('from') configuration item to the corresponding new ('to')
+    configuration item.
 
     The main difference process will use deepops.deepdiff() to work out
     what has been removed and what updated (added/changed) between the
@@ -121,7 +135,16 @@ class DiffConvert:
     cmd = None    # leave as None to cause an error if not defined in child
     ext = tuple()
     block = None
-    triggers = set()
+    trigger_blocks = set()
+
+    # _context_args_count is used by _context_args to determine the
+    # number of arguments which should be included in the context (and
+    # the rest in the 'remaining' arguments)
+    #
+    # see the __init__() constructor for more information on how they're
+    # used
+    _context_args_count = None
+    _context_args_offset = 0
 
 
     def __init__(self):
@@ -150,8 +173,43 @@ class DiffConvert:
         # store the set of indices of wildcard elements of the path (we
         # need these to get the argument list to pass to the converter
         # action methods)
-        self.wildcard_indices = {
-            i for i, v in enumerate(self._path_full) if v is None }
+        self.wildcard_indices = [
+            i for i, v in enumerate(self._path_full) if v is None ]
+
+        # initialise _context_args_count if it isn't explicitly defined
+        # (the default is None, which means 'undefined')
+        #
+        # the calculated default is usually fine but can be overridden -
+        # this is sometimes useful when writing classes covering the
+        # same commands, shared across different contexts and the
+        # context must be dynamically processed
+        #
+        # an example would be an enter() method which takes the wildcard
+        # arguments from the context and is overridden in a child class
+        if self._context_args_count is None:
+            # calculate the number of wildcard arguments in the context
+            # (elements which are None)
+            context_wildcard_count = len(
+               [ i for i, v in enumerate(self.context) if v is None ])
+
+            # the count is the number of wildcard arguments minus the
+            # 'offset' - the offset defaults to 0 so is the same but
+            # can be overridden to shift it
+            #
+            # the offset is useful when parameters need to be added or
+            # removed from this list and subclasses are generated where
+            # the context is changed by the offset remains consistent
+            self._context_args_count = (
+                context_wildcard_count + self._context_args_offset)
+
+
+    def _context_args(self, args):
+        """Split a list of wildcard arguments into a tuple - those used
+        in the context vs those used afterwards.
+        """
+
+        return (args[:self._context_args_count],
+                args[self._context_args_count:])
 
 
     def _path_matches(self, d, path):
@@ -378,20 +436,24 @@ class DiffConvert:
 
     def remove(self, old, *args):
         """The remove() method is called when the specified path is in
-        the remove differences (i.e. it's in the 'from' configuration
-        but not in the 'to' configuration), unless the containing
-        context is being removed in its entirety.
+        the remove differences (i.e. it's in the old configuration but
+        but not in the new configuration), unless the containing context
+        is being removed in its entirety.
 
         The default behaviour of this method is to call truncate() with
         the old value for 'old' and 'remove' as, if there is no specific
         behaviour for removing an entire object, we should remove the
         individual elements.
 
+        The 'new' value passed to truncate() is None as nothing exists
+        in the new configuration - if the truncate() needs this, a
+        separate remove() method will normally be needed.
+
         Keyword arguments:
 
         old -- the value of the dictionary item at the matching path in
-        the 'from' configuration dictionary (sometimes, the full details
-        of the old configuration may be required to remove it)
+        the old configuration dictionary (sometimes, the full details of
+        the old configuration may be required to remove it)
 
         *args -- [only] the arguments in the path which were matched
         with wildcards, supplied in the order they were given in the
@@ -410,13 +472,13 @@ class DiffConvert:
         differences).
         """
 
-        return self.truncate(old, old, *args)
+        return self.truncate(old, old, None, *args)
 
 
-    def truncate(self, old, rem, *args):
+    def truncate(self, old, rem, new, *args):
         """The truncate() method is identical to remove() except that it
         is called when an item is partially removed (i.e. something
-        remains in the 'to' configuration - it is 'truncated').
+        remains in the new configuration - it is 'truncated').
 
         It is useful when amending lists or sets and matching at the
         containing object level.
@@ -424,13 +486,18 @@ class DiffConvert:
         Keyword arguments:
 
         old -- the value of the dictionary item at the matching path in
-        the 'from' configuration dictionary (sometimes, the full details
-        of the old configuration may be required to remove it)
+        the old configuration dictionary (sometimes, the full details of
+        the old configuration may be required to remove it)
 
         rem -- the value of the dictionary item at the matching path in
         the remove differences dictionary (sometimes, it's necessary to
         know only what is removed from the new configuration)
 
+        new -- the value of the dictionary item at the matching path in
+        the new configuration dictionary (sometimes, parts of the old
+        configuration cannot be removed and the entire new configuration
+        is required to replace it)
+
         *args -- [only] the arguments in the path which were matched
         with wildcards, supplied in the order they were given in the
         context, cmd and ext (e.g. for a match of ["interface", None,
@@ -453,7 +520,7 @@ class DiffConvert:
 
     def add(self, new, *args):
         """The add() method is called when the specified path is in the
-        update differences but did not exist in the 'from' configuration
+        update differences but did not exist in the old configuration
         (i.e. it's something new that is being added to the
         configuration).
 
@@ -469,7 +536,7 @@ class DiffConvert:
         Keyword arguments:
 
         new -- the value of the dictionary item at the matching path in
-        the 'to' configuration dictionary
+        the new configuration dictionary
 
         *args -- [only] the arguments in the path which were matched
         with wildcards, supplied in the order they were given in the
@@ -493,13 +560,13 @@ class DiffConvert:
 
     def update(self, old, upd, new, *args):
         """The update() method is called when the specified path is in
-        the update differences and also in the 'from' configuration
-        (i.e. something is being updated in the configuration).
+        the update differences and also in the old configuration (i.e.
+        something is being updated in the configuration).
 
         Keyword arguments:
 
         old -- the value of the dictionary item at the matching path in
-        the 'from' configuration dictionary (sometimes, the old
+        the old configuration dictionary (sometimes, the old
         configuration must be removed first, before the new
         configuration can be updated, so the details are required)
 
@@ -507,8 +574,8 @@ class DiffConvert:
         the update differences dictionary
 
         new -- the value of the dictionary item at the matching path in
-        the 'to' configuration dictionary (sometimes, the full details
-        of the new configuration may be required to update it)
+        the new configuration dictionary (sometimes, the full details of
+        the new configuration may be required to update it)
 
         *args -- [only] the arguments in the path which were matched
         with wildcards, supplied in the order they were given in the
@@ -576,15 +643,17 @@ class DiffConfig:
         configuration changes that explain the differences being matched
         by the DiffConvert objects (if available).
 
-        init_dump_config=False -- dump the 'from' and 'to'
-        configurations (after exludes)
+        init_dump_config=False -- dump the old and new configurations
+        (after excludes)
 
         init_dump_diff=False -- dump the differences (remove and update
         configurations)
 
         init_debug_convert=0 -- level of debugging information for the
-        conversion process: >= 1: include steps, >= 2: include 'old/new'
-        and 'rem/upd' parameters
+        conversion process: >= 1 - include steps/class; >= 2 - include
+        'old/new' and 'rem/upd' parameters; >= 3 - include matches
+        skipped or don't return any differences; >= 4 - include
+        unmatched converters
         """
 
         # store the [initial] settings for the conversion process
@@ -876,9 +945,21 @@ class DiffConfig:
                 deepremoveitems(d, path_dict)
 
 
-    def convert(self, from_cfg, to_cfg):
-        """This method processes the conversion from the 'from'
-        configuration to the 'to' configuration, removing excluded parts
+    def _print_debug(self, msg):
+        """Print the supplied debugging message followed by a blank
+        line, if it's not empty (in which case, do nothing).
+        """
+
+        if msg:
+            for line in msg:
+                print(line)
+
+            print()
+
+
+    def convert(self, old_cfg, new_cfg):
+        """This method processes the conversion from the old
+        configuration to the new configuration, removing excluded parts
         of each and calling the applicable converters' action methods.
 
         Note that, if excludes are used, the configurations will be
@@ -899,11 +980,11 @@ class DiffConfig:
         """
 
 
-        self.apply_rules(from_cfg)
+        self.apply_rules(old_cfg)
 
         if self._dump_config:
-            print(">> 'from' configuration (after rules, if specified):",
-                  yaml.dump(deepget(dict(from_cfg),
+            print(">> old configuration (after rules, if specified):",
+                  yaml.dump(deepget(dict(old_cfg),
                                     *self._subtree_dump_filter),
                             default_flow_style=False),
                   sep="\n")
@@ -919,25 +1000,25 @@ class DiffConfig:
         # initialise the dictionary of activated triggers and their
         # configuration points
 
-        triggers = {}
+        active_triggers = {}
 
 
-        # if no 'to' config was specified, stop here (we assume we're
-        # just testing, parsing and excluding items from the 'from'
+        # if no new config was specified, stop here (we assume we're
+        # just testing, parsing and excluding items from the old
         # configuration and stopping
         #
         # we check for None explicitly for the difference between no
         # configuration and an empty configuration
 
-        if to_cfg is None:
+        if new_cfg is None:
             return None, diffs_tree
 
 
-        self.apply_rules(to_cfg)
+        self.apply_rules(new_cfg)
 
         if self._dump_config:
-            print(">> 'to' configuration (after rules, if specified):",
-                  yaml.dump(deepget(dict(to_cfg),
+            print(">> new configuration (after rules, if specified):",
+                  yaml.dump(deepget(dict(new_cfg),
                                     *self._subtree_dump_filter),
                             default_flow_style=False),
                   sep="\n")
@@ -951,8 +1032,8 @@ class DiffConfig:
         # being removed, rather than just the top of a subtree being
         # removed
 
-        remove_cfg, update_cfg = deepdiff(from_cfg, to_cfg)
-        remove_cfg_full = deepfilter(from_cfg, remove_cfg)
+        remove_cfg, update_cfg = deepdiff(old_cfg, new_cfg)
+        remove_cfg_full = deepfilter(old_cfg, remove_cfg)
 
 
         if self._dump_diff:
@@ -974,6 +1055,14 @@ class DiffConfig:
                   sep="\n")
 
 
+        # set the current block name (used to print when a new block is
+        # entered) - we use an empty string as something that won't be
+        # used as a block name (we can't use 'None' as that's the
+        # default block)
+
+        current_block = ""
+
+
         # go through the list of blocks and then the converter objects
         # within them, in order
 
@@ -993,18 +1082,21 @@ class DiffConfig:
 
 
             # if this converter is in a block which has been triggered,
-            # find all the matches in the 'to' configuration which match
+            # find all the matches in the new configuration which match
             # the context in which the trigger was set
 
             triggered_matches = []
-            if cvt.block in triggers:
-                for t in triggers[cvt.block]:
+            if cvt.block in active_triggers:
+                for t in active_triggers[cvt.block]:
                     triggered_matches.extend(
-                        cvt.explicit_context_matches(to_cfg, t))
+                        cvt.explicit_context_matches(new_cfg, t))
 
 
             # combine all the matches for this converter into a single
             # sorted list, removing duplicates
+            #
+            # we treat a 'None' key as an empty string, for sorting
+            # purposes (so it comes before everything else)
 
             all_matches = []
             for match in sorted(
@@ -1014,19 +1106,32 @@ class DiffConfig:
                     all_matches.append(match)
 
 
-            # work out if we are printing debugging for this converter
-            # (really just the 'checking difference' line at the start;
-            # we still check the debug level on individual messages)
+            # print the name of the converter if either: 1. we have the
+            # debugging messages always enabled for this, or 2. we are
+            # only debugging converters with matches and there are some
             #
-            # we record this as we use it later to determine if we need
-            # to print a blank line before the enxt one
+            # the matching converters might be skipped or not generate
+            # any differences, but we can't avoid doing that unless we
+            # store this message to be printed later, if so - it doesn't
+            # currently seem worth doing that for debugging messages
 
-            if (self._debug_convert >= 3
+            if (self._debug_convert >= DEBUG_CONVERT_NOMATCH
                 or (all_matches and self._debug_convert)):
 
+
+                # if we're entering a new block, print a message
+
+                if all_matches and (cvt.block != current_block):
+                    current_block = cvt.block
+                    print(">> [block: %s]" % current_block)
+                    print()
+
+
                 print(">>", pathstr([ "*" if i is None else i
                                     for i in cvt.get_path() ]))
 
+                print("->", repr(type(cvt)))
+
                 print()
 
 
@@ -1034,9 +1139,12 @@ class DiffConfig:
                 # handle REMOVE conversions, if matching
 
                 if match in remove_matches:
-                    if self._debug_convert:
-                        print("=> remove match:",
-                              pathstr(match, cvt.wildcard_indices))
+                    debug_msg = []
+
+                    if self._debug_convert >= DEBUG_CONVERT_STEPS:
+                        debug_msg += [
+                            "=> remove match: "
+                                + pathstr(match, cvt.wildcard_indices)]
 
 
                     # if we're removing the entire context for this
@@ -1044,15 +1152,18 @@ class DiffConfig:
                     # remove this, while it's at it
 
                     if cvt.context_removed(remove_cfg, match):
-                        if self._debug_convert:
-                            print("-> containing context being removed - skip")
-                            print()
+                        if self._debug_convert >= DEBUG_CONVERT_NODIFF:
+                            debug_msg.append(
+                                "-> containing context being removed - skip")
+
+                            self._print_debug(debug_msg)
+
                         continue
 
 
                     # check if anything remains at this level or below
-                    # in the 'to' configuration - if it does, we're
-                    # doing a partial removal
+                    # in the new configuration - if it does, we're doing
+                    # a partial removal
 
                     remove_is_truncate = False
 
@@ -1060,7 +1171,7 @@ class DiffConfig:
                         # we don't want the result here, just to find
                         # out if the path exists (if not, KeyError will
                         # be raised)
-                        deepget(to_cfg, *match, default_error=True)
+                        deepget(new_cfg, *match, default_error=True)
 
                     except KeyError:
                         # nothing remains so we're doing a full remove
@@ -1068,28 +1179,30 @@ class DiffConfig:
 
                     else:
                         # something remains so this is partial
-                        if self._debug_convert:
-                            print("-> subconfiguration not empty - truncate")
+                        if self._debug_convert >= DEBUG_CONVERT_STEPS:
+                            debug_msg += [
+                                "-> subconfiguration not empty - truncate"]
                         remove_is_truncate = True
 
 
-                    # get the 'from' and 'remove' parts of the
+                    # get the old, remove and new parts of the
                     # configuration and remove difference dictionaries,
                     # for the path specified in the converter (ignoring
                     # the extension 'ext')
 
-                    cvt_old = cvt.get_cfg(from_cfg, match)
+                    cvt_old = cvt.get_cfg(old_cfg, match)
                     cvt_rem = cvt.get_cfg(remove_cfg_full, match)
+                    cvt_new = cvt.get_cfg(new_cfg, match)
 
-                    if self._debug_convert >= 2:
-                        print("-> from configuration:",
-                              yaml.dump(cvt_old, default_flow_style=False),
-                              sep="\n")
+                    if self._debug_convert >= DEBUG_CONVERT_PARAMS:
+                        debug_msg += [
+                            "-> old configuration:",
+                            yaml.dump(cvt_old, default_flow_style=False)]
 
                         if remove_is_truncate:
-                            print("-> remove configuration:",
-                                  yaml.dump(cvt_rem, default_flow_style=False),
-                                  sep="\n")
+                            debug_msg += [
+                                "-> remove configuration:",
+                                yaml.dump(cvt_rem, default_flow_style=False)]
 
 
                     # get elements in the path matching wildcards
@@ -1098,18 +1211,20 @@ class DiffConfig:
 
 
                     # call the remove converter action method using the
-                    # corresponding entry from the 'from' configuration
+                    # corresponding entry from the old configuration
 
                     try:
                         if remove_is_truncate:
-                            diff = cvt.truncate(cvt_old, cvt_rem, *args)
+                            diff = cvt.truncate(
+                                       cvt_old, cvt_rem, cvt_new, *args)
                         else:
                             diff = cvt.remove(cvt_old, *args)
 
                     except:
-                        print("%s() with:" % ("truncate" if remove_is_truncate
-                                                  else "remove"),
-                              "  type(cvt)=" + repr(type(cvt)),
+                        print("%s.%s() with:"
+                                  % (repr(type(cvt)),
+                                     "truncate" if remove_is_truncate
+                                         else "remove"),
                               "  *args=" + repr(args),
                               sep="\n", file=sys.stderr)
 
@@ -1127,8 +1242,8 @@ class DiffConfig:
                         if isinstance(diff, str):
                             diff = [diff]
 
-                        if self._debug_convert:
-                            print("\n".join(diff))
+                        if self._debug_convert >= DEBUG_CONVERT_STEPS:
+                            debug_msg += diff
 
 
                         # add a comment, explaining the match, if
@@ -1143,7 +1258,7 @@ class DiffConfig:
                         # store this diff on the end of the list of
                         # diffs so far
 
-                        diffs.extend(diff)
+                        diffs += diff
                         diffs.append("")
 
 
@@ -1156,68 +1271,83 @@ class DiffConfig:
                         # add those to the active trigger dictionary,
                         # along with the matching patch
 
-                        for t in cvt.triggers:
+                        for t in cvt.trigger_blocks:
                             match_context = cvt.get_match_context(match)
 
-                            if self._debug_convert:
-                                print("=> trigger:",
-                                      pathstr(match_context, set()),
-                                      "block:", t)
+                            if self._debug_convert >= DEBUG_CONVERT_STEPS:
+                                debug_msg += [
+                                    "-> trigger set:",
+                                    pathstr(match_context, set()),
+                                    "block:", t]
 
-                            triggers.setdefault(t, []).append(match_context)
+                            active_triggers.setdefault(t, []).append(
+                                match_context)
 
                     else:
-                        if self._debug_convert:
-                            print("-> no action")
+                        if self._debug_convert >= DEBUG_CONVERT_NODIFF:
+                            debug_msg.append("-> no action")
+                        else:
+                            # no differences were returned by the
+                            # conversion method and we're not debugging
+                            # 'no diff's so blank any debugging message
+                            # we've built up so we don't print anything
+                            # for this match
+
+                            debug_msg = []
+
+
+                    self._print_debug(debug_msg)
 
 
                 # handle UPDATE conversions, if matching
 
                 if match in update_matches:
-                    if self._debug_convert:
-                        print("=> update match:",
-                              pathstr(match, cvt.wildcard_indices))
+                    debug_msg = []
+
+                    if self._debug_convert >= DEBUG_CONVERT_STEPS:
+                        debug_msg += [
+                            "=> update match: "
+                                + pathstr(match, cvt.wildcard_indices)]
 
 
                     args = cvt.get_wildcard_args(match)
 
 
                     # check if there is anything for this level in the
-                    # 'from' configuration - if not, we're actually
-                    # adding this, rather than updating it, so record
-                    # that
+                    # old configuration - if not, we're actually adding
+                    # this, rather than updating it, so record that
 
                     update_is_add = False
 
                     try:
-                        deepget(from_cfg, *match, default_error=True)
+                        deepget(old_cfg, *match, default_error=True)
 
                     except KeyError:
-                        if self._debug_convert:
-                            print("-> no old configuration - add")
+                        if self._debug_convert >= DEBUG_CONVERT_STEPS:
+                            debug_msg.append("-> no old configuration - add")
                         update_is_add = True
 
 
-                    # get the 'from' and 'update' parts of the
-                    # configuration and update difference dictionaries,
-                    # for the path specified in the converter (ignoring
-                    # the extension 'ext')
+                    # get the old, update and new parts of the
+                    # configuration difference dictionaries, for the
+                    # path specified in the converter (ignoring the
+                    # extension 'ext')
 
-                    cvt_old = cvt.get_cfg(from_cfg, match)
+                    cvt_old = cvt.get_cfg(old_cfg, match)
                     cvt_upd = cvt.get_cfg(update_cfg, match)
-                    cvt_new = cvt.get_cfg(to_cfg, match)
+                    cvt_new = cvt.get_cfg(new_cfg, match)
 
-                    if self._debug_convert >= 2:
+                    if self._debug_convert >= DEBUG_CONVERT_PARAMS:
                         if not update_is_add:
-                            print("-> from configuration:",
-                                  yaml.dump(cvt_old, default_flow_style=False),
-                                  "-> update configuration:",
-                                  yaml.dump(cvt_upd, default_flow_style=False),
-                                  sep="\n")
+                            debug_msg += [
+                                "-> old configuration:",
+                                yaml.dump(cvt_old, default_flow_style=False),
+                                "-> update configuration:",
+                                yaml.dump(cvt_upd, default_flow_style=False)]
 
-                        print("-> to configuration:",
-                              yaml.dump(cvt_new, default_flow_style=False),
-                              sep="\n")
+                        debug_msg += [
+                            "-> new configuration:",
+                            yaml.dump(cvt_new, default_flow_style=False)]
 
 
                     # call the update converter action method
@@ -1231,9 +1361,9 @@ class DiffConfig:
                             diff = cvt.update(cvt_old, cvt_upd, cvt_new, *args)
 
                     except:
-                        print("%s() with:" % ("add" if update_is_add
-                                                  else "update"),
-                              "  type(cvt)=" + repr(type(cvt)),
+                        print("%s.%s() with:"
+                                  % (repr(type(cvt)),
+                                     "add" if update_is_add else "update"),
                               "  *args=" + repr(args),
                               sep="\n", file=sys.stderr)
 
@@ -1246,8 +1376,8 @@ class DiffConfig:
                         if isinstance(diff, str):
                             diff = [diff]
 
-                        if self._debug_convert:
-                            print("\n".join(diff))
+                        if self._debug_convert >= DEBUG_CONVERT_STEPS:
+                            debug_msg += diff
 
 
                         if self._explain:
@@ -1261,19 +1391,26 @@ class DiffConfig:
 
                         deepsetdefault(diffs_tree, *match)
 
-                        for t in cvt.triggers:
+                        for t in cvt.trigger_blocks:
                             match_context = cvt.get_match_context(match)
 
-                            if self._debug_convert:
-                                print("=> trigger:",
-                                      pathstr(match_context, set()),
-                                      "block:", t)
+                            if self._debug_convert >= DEBUG_CONVERT_STEPS:
+                                debug_msg += [
+                                    "-> trigger set:",
+                                    pathstr(match_context, set()),
+                                    "block:", t]
 
-                            triggers.setdefault(t, []).append(match_context)
+                            active_triggers.setdefault(t, []).append(
+                                match_context)
 
                     else:
-                        if self._debug_convert:
-                            print("-> no action")
+                        if self._debug_convert >= DEBUG_CONVERT_NODIFF:
+                            debug_msg.append("-> no action")
+                        else:
+                            debug_msg = []
+
+
+                    self._print_debug(debug_msg)
 
 
                 # handle TRIGGERED conversions, if matching, and this
@@ -1283,32 +1420,35 @@ class DiffConfig:
                     and (match not in remove_matches)
                     and (match not in update_matches)):
 
-                    if self._debug_convert:
-                        print("=> trigger match:",
-                              pathstr(match, cvt.wildcard_indices),
-                              "block:", cvt.block)
+                    debug_msg = []
+
+                    if self._debug_convert >= DEBUG_CONVERT_STEPS:
+                        debug_msg += [
+                            "=> trigger match: "
+                                + pathstr(match, cvt.wildcard_indices),
+                            "block:", cvt.block]
 
 
                     args = cvt.get_wildcard_args(match)
 
 
-                    # get the 'from' and 'update' parts of the
-                    # configuration and update difference dictionaries,
-                    # for the path specified in the converter (ignoring
-                    # the extension 'ext')
+                    # get the old, update and new parts of the
+                    # configuration difference dictionaries for the path
+                    # specified in the converter (ignoring the extension
+                    # 'ext')
 
-                    cvt_old = cvt.get_cfg(from_cfg, match)
+                    cvt_old = cvt.get_cfg(old_cfg, match)
                     cvt_upd = cvt.get_cfg(update_cfg, match)
-                    cvt_new = cvt.get_cfg(to_cfg, match)
+                    cvt_new = cvt.get_cfg(new_cfg, match)
 
-                    if self._debug_convert >= 2:
-                        print("-> from configuration:",
-                              yaml.dump(cvt_old, default_flow_style=False),
-                              "-> update configuration:",
-                              yaml.dump(cvt_upd, default_flow_style=False),
-                              "-> to configuration:",
-                              yaml.dump(cvt_new, default_flow_style=False),
-                              sep="\n")
+                    if self._debug_convert >= DEBUG_CONVERT_PARAMS:
+                        debug_msg += [
+                            "-> old configuration:",
+                            yaml.dump(cvt_old, default_flow_style=False),
+                            "-> update configuration:",
+                            yaml.dump(cvt_upd, default_flow_style=False),
+                            "-> new configuration:",
+                            yaml.dump(cvt_new, default_flow_style=False)]
 
 
                     # call the trigger converter action method
@@ -1317,8 +1457,7 @@ class DiffConfig:
                         diff = cvt.trigger(cvt_old, cvt_upd, cvt_new, *args)
 
                     except:
-                        print("trigger() with:",
-                              "  type(cvt)=" + repr(type(cvt)),
+                        print("%s.trigger() with:" % repr(type(cvt)),
                               "  *args=" + repr(args),
                               sep="\n", file=sys.stderr)
 
@@ -1331,8 +1470,8 @@ class DiffConfig:
                         if isinstance(diff, str):
                             diff = [diff]
 
-                        if self._debug_convert:
-                            print("\n".join(diff))
+                        if self._debug_convert >= DEBUG_CONVERT_STEPS:
+                            debug_msg += diff
 
 
                         if self._explain:
@@ -1349,14 +1488,13 @@ class DiffConfig:
                         # except one trigger cannot fire another
 
                     else:
-                        if self._debug_convert:
-                            print("-> no action")
+                        if self._debug_convert >= DEBUG_CONVERT_STEPS:
+                            debug_msg.append("-> no action")
+                        else:
+                            debug_msg = []
 
 
-                # print a blank line before the next match or converter
-
-                if self._debug_convert:
-                    print()
+                    self._print_debug(debug_msg)
 
 
         # if nothing was generated, just return nothing
